module Auxiliary exposing (..)

import Html exposing (Html)
import Html.Attributes
import Set exposing (Set)
import Types exposing (..)


boolTreeToString : Formula -> String
boolTreeToString tree =
    case tree of
        And _ _ ->
            "∧"

        Or _ _ ->
            "∨"

        Not _ ->
            "¬"

        Const val ->
            boolToString val

        Var var ->
            var


getVariables : Formula -> Set String
getVariables formula =
    case formula of
        Const _ ->
            Set.empty

        Var x ->
            Set.fromList [ x ]

        And f1 f2 ->
            Set.union (getVariables f1) (getVariables f2)

        Or f1 f2 ->
            Set.union (getVariables f1) (getVariables f2)

        Not f1 ->
            getVariables f1



{--
    Takes the first elements from parsing result
--}


extractResult : Result (List StackElement) Formula -> Formula
extractResult res =
    let
        filter : StackElement -> Maybe Formula
        filter stack =
            case stack of
                Formula form ->
                    Just form

                _ ->
                    Nothing
    in
    case res of
        Err stack ->
            case List.filterMap filter stack of
                [] ->
                    Var "Smth went wrong while parsing"

                a :: _ ->
                    a

        Ok formula ->
            formula



{--
    Takes all partial Formulas from parsing result
--}


extractStack : Result (List StackElement) Formula -> List Formula
extractStack res =
    let
        filter : StackElement -> Maybe Formula
        filter stack =
            case stack of
                Formula form ->
                    Just form

                Token _ ->
                    Nothing
    in
    case res of
        Err stack ->
            let
                filtered =
                    List.filterMap filter stack
            in
            case filtered of
                [] ->
                    [ Var "Smth went wrong while parsing" ]

                _ :: _ ->
                    filtered

        Ok formula ->
            [ formula ]


boolToString : Bool -> String
boolToString b =
    if b then
        "T"

    else
        "F"
