module MealyMachine exposing (Mealy, deltaHat, parse, parseAndMerge)





type alias Mealy state outputAlphabet =
    { startState : state
    , transition : state -> Char -> (state, outputAlphabet)
    , acceptingStates : List (state)
    }


deltaHat : Mealy state outputAlphabet -> state -> List Char -> state
deltaHat machine state list =
    case list of
        [] ->
            state

        c :: rest ->
            deltaHat machine (Tuple.first (machine.transition state c)) rest


lambdaHat : Mealy state outputAlphabet -> state -> List Char -> List (Char, outputAlphabet)
lambdaHat machine state list =
    case list of
        [] ->
            []
        c :: rest ->
            (c , Tuple.second (machine.transition state c)) :: lambdaHat machine (Tuple.first (machine.transition state c)) rest




parse : Mealy state outputAlphabet -> String -> List (Char, outputAlphabet)
parse machine inputString =
    lambdaHat machine machine.startState (String.toList inputString) 


parseAndMerge : Mealy state outputAlphabet -> String -> List (String, outputAlphabet)
parseAndMerge machine inputString =
    condense (convertList (parse machine inputString))






sameOutput : (String, outputAlphabet) -> (String, outputAlphabet) -> Bool
sameOutput tupl1 tupl2 =
    if (Tuple.second tupl1) == (Tuple.second tupl2) then
        True
    else
        False



convertList : List (Char, outputAlphabet) -> List (String, outputAlphabet)
convertList list =
    List.map (\(c, s) -> (String.fromChar c, s)) list



condense : List (String, outputAlphabet) -> List (String, outputAlphabet)
condense list =
    case list of
        [] ->
            []
        c :: rest ->
            appendOrMerge c (condense rest)






-- use the List version again

appendOrMerge : (String, outputAlphabet) -> List (String, outputAlphabet) -> List (String, outputAlphabet)
appendOrMerge tupl list =
    case list of
        [] -> 
            List.singleton tupl
        c :: rest ->
            if sameOutput tupl c then
                ((Tuple.first tupl) ++ (Tuple.first c) ,
                (Tuple.second tupl)) :: rest
            else
                tupl :: list




                            

-- merges the input String if output isnt "end"
append : (String , String) -> (String, String) -> (String, String)
append tupl1 tupl2 =
    if (Tuple.second tupl2) == "end" then
        tupl1
    else
        ((Tuple.first tupl1) ++ (Tuple.first tupl2) ,
        (Tuple.second tupl1))
   
    