module Types exposing (..)

import Dict exposing (Dict)


type StackElement
    = Formula Formula
    | Token BoolToken


type Formula
    = Const Bool
    | Var String
    | And Formula Formula
    | Or Formula Formula
    | Not Formula


type BoolToken
    = CONST Bool
    | VAR String
    | AND
    | OR
    | NOT
    | OPEN
    | CLOSE


type alias TruthTableEntry =
    List ( String, Bool, Formula )


type alias TruthTable =
    List TruthTableEntry


type alias Solution =
    { name : String
    , toString : Formula -> String
    , evaluate : Formula -> Dict String Bool -> Formula
    , getAllAssignments : List String -> List (List ( String, Bool ))
    , generateTruthTable : List String -> Formula -> List ( List ( String, Bool ), Formula )
    , parse : List BoolToken -> Result (List StackElement) Formula
    }


example : List BoolToken
example =
    [ OPEN, VAR "x", OR, OPEN, OPEN, NOT, VAR "y", CLOSE, AND, VAR "z", CLOSE, CLOSE ]
